package lightsout;

import java.util.ArrayList;

/** DFS solution to the Lights out problem. */
public class DfsSolution {

	private static boolean explore(Board board, ArrayList<String> moves) {		
		// Check whether the board is solved.
		if (board.isSolved()) {
			moves.add(board.toString());
			return true;
		}
		
		// Try all possible presses.
		String prevBoard = board.toString();
		for (int row = 0; row < board.getRows(); ++row) {
			for (int col = 0; col < board.getCols(); ++col) {
				// Try to press a switch only if it has not been pressed.
				if (!board.isPressed(row, col)) {
					board.press(row, col);
					
					boolean ans = explore(board, moves);
					if (ans) {
						moves.add(prevBoard + "\nPressed row : " + row + " and col: " + col);
						return true;
					}
					
					// Restore the previous states.
					board.undoPress(row, col);
				}
			}
		}
		return false;
	}
	
	private static Board get4x4Board() {
		Board board = new Board(4, 4);
		board.set(0, 0, Board.ON);
		board.set(0, 1, Board.ON);
		board.set(0, 2, Board.ON);
		board.set(1, 1, Board.ON);
		board.set(1, 2, Board.ON);
		board.set(2, 0, Board.ON);
		board.set(2, 1, Board.ON);
		board.set(2, 3, Board.ON);
		board.set(3, 0, Board.ON);
		board.set(3, 3, Board.ON);
		return board;
	}
	
	private static Board get5x5Board() {
		Board board = new Board(5, 5);
		board.set(1, 0, Board.ON);
		board.set(2, 0, Board.ON);
		board.set(2, 3, Board.ON);
		board.set(3, 1, Board.ON);
		board.set(3, 3, Board.ON);
		board.set(4, 1, Board.ON);
		board.set(4, 4, Board.ON);
		return board;
	}
	
	public static void main(String[] args) {
		Board board = get4x4Board();
		ArrayList<String> moves = new ArrayList<>();
		boolean ans = explore(board, moves);
		
		if (ans) {
			System.out.println("Found a solution.");
			for (String move : moves) {
				System.out.println(move);
			}
		} else {
			System.out.println("Did not find a solution.");
		}
	}
}
